var utils_1 = require('../libs/utils');
var cli_colors_util_1 = require('./cli-colors.util');
'log' | 'error' | 'warn' | 'debug' | 'verbose';
var Logger = (function () {
    function Logger(context, readonly) {
        if (readonly === void 0) { readonly = isTimestampEnabled = false; }
        this.context = context;
        this.readonly = readonly;
    }
    Logger.overrideLogger = function (logger) {
        if (Array.isArray(logger)) {
            this.logLevels = logger;
            return;
        }
        this.instance = utils_1.isObject(logger) ? (logger) : as;
        ILogger;
        undefined;
    };
    Logger.log = function (message, context, isTimeDiffEnabled) {
        if (context === void 0) { context = ''; }
        if (isTimeDiffEnabled === void 0) { isTimeDiffEnabled = true; }
        this.printMessage(message, cli_colors_util_1.clc.green, context, isTimeDiffEnabled);
    };
    Logger.error = function (message, trace, context, isTimeDiffEnabled) {
        if (trace === void 0) { trace = ''; }
        if (context === void 0) { context = ''; }
        if (isTimeDiffEnabled === void 0) { isTimeDiffEnabled = true; }
        this.printMessage(message, cli_colors_util_1.clc.red, context, isTimeDiffEnabled, 'stderr');
        this.printStackTrace(trace);
    };
    Logger.warn = function (message, context, isTimeDiffEnabled) {
        if (context === void 0) { context = ''; }
        if (isTimeDiffEnabled === void 0) { isTimeDiffEnabled = true; }
        this.printMessage(message, cli_colors_util_1.clc.yellow, context, isTimeDiffEnabled);
    };
    Logger.debug = function (message, context, isTimeDiffEnabled) {
        if (context === void 0) { context = ''; }
        if (isTimeDiffEnabled === void 0) { isTimeDiffEnabled = true; }
        this.printMessage(message, cli_colors_util_1.clc.magentaBright, context, isTimeDiffEnabled);
    };
    Logger.verbose = function (message, context, isTimeDiffEnabled) {
        if (context === void 0) { context = ''; }
        if (isTimeDiffEnabled === void 0) { isTimeDiffEnabled = true; }
        this.printMessage(message, cli_colors_util_1.clc.cyanBright, context, isTimeDiffEnabled);
    };
    Logger.getTimestamp = function () {
        var localeStringOptions = {
            year: 'numeric',
            hour: 'numeric',
            minute: 'numeric',
            second: 'numeric',
            day: '2-digit',
            month: '2-digit'
        };
        return new Date(Date.now()).toLocaleString(undefined, localeStringOptions);
    };
    Logger.printMessage = function (message, color, context, isTimeDiffEnabled, writeStreamType) {
        if (context === void 0) { context = ''; }
        if (writeStreamType === void 0) { writeStreamType =  | 'stderr'; }
        var output = utils_1.isObject(message)
            ? color('Object:') + "\n" + JSON.stringify(message, null, 2) + "\n"
            : color(message);
        var pidMessage = color("[Nest] " + process.pid + "   - ");
        var contextMessage = context ? cli_colors_util_1.yellow("[" + context + "] ") : '';
        var timestampDiff = this.updateAndGetTimestampDiff(isTimeDiffEnabled);
        var instance = (this.instance), as = typeof Logger, Logger;
        var computedMessage = "" + pidMessage + instance.getTimestamp() + "   " + contextMessage + output + timestampDiff + "\n";
        process[writeStreamType ?  ? 'stdout' :  : ].write(computedMessage);
    };
    Logger.updateAndGetTimestampDiff = function (isTimeDiffEnabled) {
        var includeTimestamp = Logger.lastTimestamp && isTimeDiffEnabled;
        var result = includeTimestamp ? cli_colors_util_1.yellow(" +" + (Date.now() - Logger.lastTimestamp) + "ms") : '';
        Logger.lastTimestamp = Date.now();
        return result;
    };
    Logger.printStackTrace = function (trace) {
        if (!trace) {
            return;
        }
        process.stderr.write(trace + "\n");
    };
    Logger.prototype.error = function (message, trace, context) {
        if (trace === void 0) { trace = ''; }
        var instance = this.getInstance();
        if (!this.isLogLevelEnabled('error')) {
            return;
        }
        instance && instance.error.call(instance, message, trace, context || this.context);
    };
    Logger.prototype.log = function (message, context) {
        this.callFunction('log', message, context);
    };
    Logger.prototype.warn = function (message, context) {
        this.callFunction('warn', message, context);
    };
    Logger.prototype.debug = function (message, context) {
        this.callFunction('debug', message, context);
    };
    Logger.prototype.verbose = function (message, context) {
        this.callFunction('verbose', message, context);
    };
    Logger.prototype.setContext = function (context) {
        this.context = context;
    };
    Logger.prototype.getTimestamp = function () {
        return Logger.getTimestamp();
    };
    Logger.prototype.getInstance = function () {
        var instance = Logger.instance;
        return instance === this ? Logger : instance;
    };
    Logger.prototype.callFunction = function (name, message, context) {
        if (name === void 0) { name =  | 'warn' | 'debug' | 'verbose'; }
        if (!this.isLogLevelEnabled(name)) {
            return;
        }
        var instance = this.getInstance();
        var func = instance && (instance), as = typeof Logger, name = (void 0)[0];
        func && func.call(instance, message, context || this.context, this.isTimestampEnabled);
    };
    Logger.prototype.isLogLevelEnabled = function (level) {
        return Logger.logLevels.includes(level);
    };
    Logger.instance = Logger;
    Logger.logLevels = ['log', 'error', 'warn', 'debug', 'verbose'];
    return Logger;
})();
exports.Logger = Logger;
